/*
 *  ******************************************************************************
 *  *
 *  *
 *  * This program and the accompanying materials are made available under the
 *  * terms of the Apache License, Version 2.0 which is available at
 *  * https://www.apache.org/licenses/LICENSE-2.0.
 *  *
 *  *  See the NOTICE file distributed with this work for additional
 *  *  information regarding copyright ownership.
 *  * Unless required by applicable law or agreed to in writing, software
 *  * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 *  * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 *  * License for the specific language governing permissions and limitations
 *  * under the License.
 *  *
 *  * SPDX-License-Identifier: Apache-2.0
 *  *****************************************************************************
 */
package org.deeplearning4j.nn.layers;

import org.deeplearning4j.BaseDL4JTest;
import org.deeplearning4j.nn.layers.convolution.ConvolutionHelper;
import org.deeplearning4j.nn.layers.convolution.subsampling.SubsamplingHelper;
import org.deeplearning4j.nn.layers.mkldnn.*;
import org.deeplearning4j.nn.layers.normalization.BatchNormalizationHelper;
import org.deeplearning4j.nn.layers.normalization.LocalResponseNormalizationHelper;
import org.deeplearning4j.nn.layers.recurrent.LSTMHelper;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.nd4j.linalg.api.buffer.DataType;

import static org.junit.jupiter.api.Assertions.assertNotNull;

/**
 */
@DisplayName("Activation Layer Test")
public class HelperUtilsTest extends BaseDL4JTest {

    @Override
    public DataType getDataType() {
        return DataType.FLOAT;
    }

    @Test
    @DisplayName("Test instance creation of various helpers")
    public void testOneDnnHelperCreation() {
        assertNotNull(HelperUtils.createHelper("",
                MKLDNNLSTMHelper.class.getName(), LSTMHelper.class,"layername",getDataType()));
        assertNotNull(HelperUtils.createHelper("", MKLDNNBatchNormHelper.class.getName(),
                BatchNormalizationHelper.class,"layername",getDataType()));
        assertNotNull(HelperUtils.createHelper("", MKLDNNLocalResponseNormalizationHelper.class.getName(),
                LocalResponseNormalizationHelper.class,"layername",getDataType()));
        assertNotNull(HelperUtils.createHelper("", MKLDNNSubsamplingHelper.class.getName(),
                SubsamplingHelper.class,"layername",getDataType()));
        assertNotNull(HelperUtils.createHelper("", MKLDNNConvHelper.class.getName(),
                ConvolutionHelper.class,"layername",getDataType()));


    }


}
